import time
from datetime import datetime, timedelta

import serial
import MySQLdb as mdb
import sys

USB = '/dev/ttyUSB0'

################## Funcions ########################

def comprovarVariablesBaseDeDades(variablesActuals, dbColumns):
    #mira si totes les variables que volem estan a la BD 'xbee' taula 'dades'
    #return : llista amb variables que falten

    columnes = []
    for col in dbColumns:
        columnes.append(col[0].lower())

    faltenVariables = []
    for var in variablesActuals:
        if var not in columnes:
            # print var
            faltenVariables.append(var)

    return faltenVariables


def llegirFitxerVariables():
    #llegim en el fitxer de cofiguracio les vaiables que tenim
    variablesActuals = []
    with open('variables.txt') as file:
        for line in file:
            variablesActuals.append(line.replace("\n", "").lower())  # borrem el salt de linia, no forma part de la variable
    return variablesActuals


def separarCoordenades(coordenades, incoming):
    # separem les coordenades en variables independents

    coordenades = coordenades.split(",")  # separem les coordenades xyz
    for variable in coordenades:
        incoming.append(variable)
    return incoming


def parseIncommingString(incoming, variables):
    # separem les dades que ens arriben en les diferents variables

    myMap = {}

    start = incoming.find("-# ")  # busquem el principi de la cadena
    end = incoming.find(" -#")  # busquem el final de la cadena
    incoming = incoming[start+3:end]  # retallem els caracters d'inici i desem la resta de la cadena.
    incoming = incoming.split(" -")  # separem la cadena per el caracter separador

    dada = ""

    for var in variables:
        for data in incoming:

            parell = data.split(":")  # separem el parell variable:dada
            variable = parell[0].replace("-", "").lower()  # esborrem el '-' i ho passem a minuscules
            if len(parell) > 1:  # si data era del format 'xxxx:yyyy' i no falta la dada 'xxxx:'
                dada = parell[1].replace(" ", "")

            if var == variable:
                myMap.update({var: dada})  # el dictionaty(hashMap) ens evita duplicats.!!!!

    if myMap:
        myMap = afegirDates(myMap)

    return myMap


def afegirDates(myMap):
    # afegeix les dates(date i dateOK) a les dades.

    now = datetime.now()
    roundedDate = roundDate(now)

    myMap.update({"date": now.strftime('%Y-%m-%d %H:%M:%S')})
    myMap.update({"dateOK": roundedDate.strftime('%Y-%m-%d %H:%M:%S')})

    return myMap


def roundDate(date):
    # arrodoneix la data que a l'hora mes propera

    minuts = date.minute
    if minuts >= 30:
        date = date + timedelta(hours=1)  # sumem 1 hora

    date = date.replace(minute=0)  # posem minuts a 0
    date = date.replace(second=0)  # posem segons a 0

    return date


def afegirVariablesADB(variablesAfegirDB, cur):
    # Afegim les variables que falten a la base de dades
    # aquest pas ens serveix per afegir variables des de l'arxiu de configuracio de variables(variables.txt)
    # les variables afegides des de l'arxiu de variables, seran de tipus text (varchar)
    # si es vol afegir variables d'un tipus concret, s'ha de fer des de phpmyadmin.

    for var in variablesAfegirDB:
        cur.execute("ALTER TABLE dades ADD "+var+" VARCHAR(60);")
    return


def desarDades(cur, dades):
    # es desen les dades a la Base de Dades

    # a la variable nomColumnes, posaren el nom de la variable(del camp de la base de dades)
    nomColumnes = ""
    # a la variable dadesDeColumnes posarem el valor de la variable
    dadesDeColumnes = ""
    for data in dades:
        nomColumnes = nomColumnes+data+", "
        # IMPORTANT ** les ' ' per que accepti caracters especials
        dadesDeColumnes = dadesDeColumnes+"'"+dades.get(data)+"', "

    nomColumnes = nomColumnes[:-2]  # treure els 2 ultims simbols  ', '
    dadesDeColumnes = dadesDeColumnes[:-2]


    print "Inserim a la BD: "+"INSERT INTO dades ("+nomColumnes+") VALUES ("+dadesDeColumnes+")"
    if dades:  # com en c o c++, si retorna !=0 vol dir que hi ha algo
        cur.execute("INSERT INTO dades ("+nomColumnes+") VALUES ("+dadesDeColumnes+")")
    return


def comprovarVariables(cur, variables):
    # mirem les variables que tenim a la base de dades
    # i afegim les variables que falten

    cur.execute("SELECT column_name FROM information_schema.columns WHERE table_name = 'dades' ;")
    columns = cur.fetchall()

    variablesAfegirDB = comprovarVariablesBaseDeDades(variables, columns)

    afegirVariablesADB(variablesAfegirDB, cur)
    return


def afegirDadesQueFalten(cur, mac):
    # En cas que falti una dada ( 15:00, 16:00, 18:00 ....) aqui faltaria les dades de les 17:00.

    now = datetime.now()
    cur.execute("select dateOK from dades where mac = '"+str(mac)+"' order by id desc limit 1;")
    row = cur.fetchone()

    if row is not None and row[0] is not None:

        ultimaHora = row[0]
        diff = now-ultimaHora
        diferencia = int(round(diff.total_seconds()/3600)) # arrodonim i passem a enter.

        for i in range(1, diferencia):
            novaData = ultimaHora + timedelta(hours=i)
            cur.execute('INSERT INTO dades (mac,dateOK) VALUES(%s,%s)', (mac, novaData.strftime('%Y-%m-%d %H:%M:%S'),))



###################################################-
################## Funcions #######################-
###################################################-



con = mdb.connect('localhost', 'xbee', 'xbee1234', 'xbee')
con.autocommit(True)  # or con.commit() afetr any insert
cur = con.cursor()


variables = llegirFitxerVariables()
comprovarVariables(cur, variables)

totalArguments = len(sys.argv)

if totalArguments > 1:
    USB = sys.argv[1]

try:
    ser = serial.Serial(USB, 38400)
except:
    print datetime.now()
    print "Error: no s'ha trobat l'USB"
    raise

start = time.time()
end = time.time()

while True:

    if end-start > 600:  # per comprovar que no s'han afegit variables noves al fitxer de variables.
        variables = llegirFitxerVariables()
        comprovarVariables(cur, variables)
        print "entrat en comprovacio de variables"
        start = time.time()

    try:
        incoming = ser.readline().strip()
    except:
        print "error en llegir del USB, seguim"
        incoming = ""
        pass

    print "Trama rebuda: "+incoming
    dades = parseIncommingString(incoming, variables)
    mac = dades.get('mac')

    if mac is not None:  # evita trames tallades ( sobre tot al inici) -mac sempre estara al principi
        afegirDadesQueFalten(cur, mac)
        desarDades(cur, dades)  # tambe s'hi ha de posar la data i hora.

    end = time.time()
    incoming = ""

con.close()




# DELIMITER //
# DROP TRIGGER IF EXISTS dateOK_insert_trigger//
# CREATE TRIGGER dateOK_insert_trigger
# BEFORE INSERT ON dades
# FOR EACH ROW
# BEGIN
# SET NEW.dateOK = DATE(NOW() + INTERVAL 30 MINUTE) + INTERVAL HOUR(NOW() + INTERVAL 30 MINUTE) HOUR;
# END;//
# DELIMITER ;


# DELIMITER //
# DROP TRIGGER IF EXISTS dateOK_insert_trigger//
# CREATE TRIGGER dateOK_insert_trigger
# BEFORE INSERT ON dades
# FOR EACH ROW
# BEGIN
# SET NEW.dateOK = DATE(NEW.date + INTERVAL 30 MINUTE) + INTERVAL HOUR(NOW() + INTERVAL 30 MINUTE) HOUR;
# END;//
# DELIMITER ;
