/*  
 *  ------  GPRS_20 - Uploading files to a FTP server  -------- 
 *  
 *  Explanation: This example shows how to upload a file to a FTP server
 *  
 *  Copyright (C) 2013 Libelium Comunicaciones Distribuidas S.L. 
 *  http://www.libelium.com 
 *  
 *  This program is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  This program is distributed in the hope that it will be useful, 
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 *  GNU General Public License for more details. 
 *  
 *  You should have received a copy of the GNU General Public License 
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. 
 *  
 *  Version:           0.3 
 *  Design:            David Gascón 
 *  Implementation:    Alejandro Gállego
 */

const char filename[]="/fileSD2.txt";

int answer;

void setup()
{
  
  SD.ON();
   if(SD.del(filename)) 
  {
    USB.println(F("file deleted"));
  }
  else 
  {
    USB.println(F("file NOT deleted"));  
  }

  // Create file
  if(SD.create(filename))
  {
    USB.println(F("file created"));
  }
  else 
  {
    USB.println(F("file NOT created"));  
  } 
 
  SD.appendln(filename,"00000000000000000000000000000000000000000000000000000000000");
  SD.appendln(filename,"11111111111111111111111111111111111111111111111111111111111");
  SD.appendln(filename,"22222222222222222222222222222222222222222222222222222222222");
  SD.appendln(filename,"33333333333333333333333333333333333333333333333333333333333");
  SD.appendln(filename,"44444444444444444444444444444444444444444444444444444444444");
  SD.appendln(filename,"55555555555555555555555555555555555555555555555555555555555");
  SD.appendln(filename,"66666666666666666666666666666666666666666666666666666666666");
  SD.appendln(filename,"77777777777777777777777777777777777777777777777777777777777");
  SD.appendln(filename,"88888888888888888888888888888888888888888888888888888888888");
  SD.appendln(filename,"99999999999999999999999999999999999999999999999999999999999"); 
  
}

void loop()
{
    
    // setup for Serial port over USB:
    USB.begin();
    USB.println(F("USB port started..."));
    
    // activates the GPRS_Pro module:
    answer = GPRS_Pro.ON(); 
    if ((answer == 1) || (answer == -3))
    {
        USB.println(F("GPRS_Pro module ready..."));
             
        // sets pin code:
        USB.println(F("Setting PIN code..."));
        // **** must be substituted by the SIM code
        if (GPRS_Pro.setPIN("****") == 1) 
        {
            USB.println(F("PIN code accepted"));
        }
        else
        {
            USB.println(F("PIN code incorrect"));
        }
    
        // waits for connection to the network:
        answer = GPRS_Pro.check(180);    
        if (answer == 1)
        {
            USB.println(F("GPRS_Pro module connected to the network..."));
                
            // configures GPRS Connection for HTTP or FTP applications:
            answer = GPRS_Pro.configureGPRS_HTTP_FTP(1);
            if (answer == 1)
            {
                
                USB.println(F("Uploading the file..."));
                
                // uploads file from SD card to the FTP server:
                answer = GPRS_Pro.uploadFile(filename, "/fileFTP2.txt", "tgprs@libelium.com", "ftp1234", "pruebas.libelium.com", "21", 1);
                if (answer == 1)
                {
                    USB.println(F("Upload done"));
                }
                else if (answer < -40)
                {
                    USB.print(F("Upload failed. Error code: "));
                    USB.println(answer, DEC);
                    USB.print(F("CME error code: "));
                    USB.println(GPRS_Pro.CME_CMS_code, DEC);
                }
                else 
                {
                    USB.print(F("Upload failed. Error code: "));
                    USB.println(answer, DEC);
                }
            }
            else
            {
                USB.println(F("Configuration failed. Error code:"));
                USB.println(answer, DEC);
            }
        }
        else
        {
            USB.println(F("GPRS_Pro module cannot connect to the network"));     
        }
    }
    else
    {
        USB.println(F("GPRS_Pro module not ready"));    
    }
    
    GPRS_Pro.OFF(); // powers off the GPRS_Pro module
    
    USB.println(F("Sleeping..."));
    
    // activates the RTC:
    RTC.ON();
    // sleeps one hour
    PWR.deepSleep("00:01:00:00", RTC_OFFSET, RTC_ALM1_MODE1, ALL_OFF);
    
    // deactivates the RTC:
    RTC.OFF();
}
