/*  
 *  ------ RFID/NFC Basic Example --------
 *  
 *  Explanation: This sketch shows how to use the most important features
 *  of the RFID/NFC module in Waspmote. First, the program looks for tags.
 *  After a tag was found, the program tries to authenticate a sector and
 *  writes in one block. Finally, it reads the content of that block.
 *  
 *  Copyright (C) 2012 Libelium Comunicaciones Distribuidas S.L.
 *  http://www.libelium.com
 *  
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  Version:        0.1
 *  Design:        David Gascon
 *  Implementation: Ahmad Saad, Javier Solobera
 */



// stores the 16 bytes data to be written in a block:
blockData writeData = {0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                       0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x01};
// stores the 16 bytes data read from a block:
blockData readData;
// stores the key or password:
uint8_t keyAccess[] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF };
// stores the block address:
uint8_t address = 0x04;
// stores the UID (unique identifier) of a card:
CardIdentifier CardID;
// stores the answer to request:
uint8_t ATQ[2];



void setup()
{
  // switchs on the RFID/NFC @ 13.56 MHz module, type B, and asigns the socket
  RFID13.ON(SOCKET0, B);
  delay(1000);
  
  USB.begin();
  USB.print("\n");
  USB.println("RFID/NFC @ 13.56 MHz module started");
}



void loop()
{
  USB.print("\n");
  USB.println("Ready to read...");
  
  // get the UID
  RFID13.init(CardID, ATQ);
  USB.print("\n");
  USB.print("the Card UID: ");
  RFID13.print(CardID, 4);
  // auntenticate block number 4 with its access key (2nd sector)
  if (!RFID13.authenticate(CardID, address, keyAccess))
  {
    USB.print("\n");
    USB.println("Authentication failed");
  }
  else  // success
  {
    USB.print("\n");
    USB.println("Authentication OK");
    // after authentication, write blockData in the block
    if (!RFID13.writeData(address, writeData))
    {
      USB.print("\n");
      USB.println("Write failed");
    }
    else  // success
    {
      USB.print("\n");
      USB.println("Write block OK");
      // read from address after write
      if (!RFID13.readData(address, readData))
      {
        USB.print("\n");
        USB.println("Read failed");
      }
      else  // success
      {
        USB.print("\n");
        USB.println("Read block OK");
        USB.print("Data read: ");
        RFID13.print(readData, 16);
      }
    }
  }
  USB.print("\n");
  delay(1000); // wait some time each loop
    
  
}
