/*  
 *  ------ Password Simple --------
 *  
 *  Explanation: This sketch changes the keys and access conditions in a
 *  block and checks the block can be accessed with the new conditions.
 *  Then changes the keys and access conditions to the original values and
 *  checks again.
 *  
 *  Copyright (C) 2012 Libelium Comunicaciones Distribuidas S.L.
 *  http://www.libelium.com
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 *  Version:                0.1
 *  Design:                 David Gascon
 *  Implementation:         Javier Solobera, Ahmad Saad
 */



// stores the status of the executed command:
short state;
// auxiliar buffer:
unsigned char aux[16];
// stores the UID (unique identifier) of a card:
unsigned char CardID[4];
// stores the key or password:
unsigned char keyOld[]= {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};

unsigned char keyA[]= {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};

unsigned char keyB[]= {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};

unsigned char config[] = {0xFF, 0x07, 0x80, 0x69}; // only edit if strong knowledge about the RFID/NFC module

int address = 27;


void setup()
{
  // switchs on the RFID/NFC @ 13.56 MHZ module, type B, and asigns the socket
  RFID13.ON(SOCKET0, B);
  delay(1000);
  
  USB.begin();
  USB.println("RFID/NFC @ 13.56 MHz module started");
}



void loop()
{
  USB.print("\r\n++++++++++++++++++++++++++++++++++++");
  // **** init the RFID/NFC reader
  state = RFID13.init(CardID, aux);
    if (aux[0] == aux[1]) // if so, there is no card on the EM field
    {
      USB.print("\r\nRequest error - no card found");
    }
    else // a card was found
    {
      USB.print("\r\nRequest | Answer: ");
      RFID13.print(aux, 2);  // show the ATQ
      USB.print("\r\nAnticollSelect: ");
      USB.print(state, HEX); // show the status of the executed init command (should be 0)
      USB.print(" | Answer: ");
      RFID13.print(CardID, 4);  // show the UID (Unique IDentifier) of the read card (4 bytes)
      // ###### write new key in block 3/7/11/15; then authenticate this address and get the new value
      state = RFID13.setKeys(CardID, keyOld, keyA, keyB, config, aux, address);
      USB.print("\r\n**write new key, state: ");
      USB.print(state, HEX);
      if (state == 1)
      {
        USB.print(" --> correct key change ");
      }
      else
      {
        USB.print(" --> ** ERROR: keys not changed ** ");
      }
      // ###### check block 3/7/11/15 (the keys will return 0s)
      // **** authenticate again, now with the new key A
      state = RFID13.authenticate(CardID, address , keyA);
      USB.print("\r\n**Authentication block: ");
      USB.print(state, HEX);
      if (state == 1)
      {
        USB.print(" --> correct authentication ");
      }
      else
      {
        USB.print(" --> ** ERROR: bad authentication ** ");
      }
      // **** read the content of the new control block
      state = RFID13.readData(address ,aux);
      USB.print("\r\n**Read trailer block:   ");
      USB.print(state, HEX);
      USB.print(" | Answer: ");
      RFID13.print(aux, 16);
      // ####### check block 1/5/9/13
      // **** authenticate again, now with the new key A
      state = RFID13.authenticate(CardID,address-1, keyA);
      USB.print("\r\n**Authentication block: ");
      USB.print(state, HEX);
      // **** read block
      state = RFID13.readData(address-1 , aux);
      USB.print("\r\n**Read mem block, state:  ");
      USB.print(state, HEX);
      USB.print(" | Answer: ");
      if (state == 1)
      {
        RFID13.print(aux, 16);
      }
      // ###### write old, original key in block 3/7/11/15; then authenticate this address and get the new value
      state = RFID13.setKeys(CardID, keyA, keyOld, keyB, config, aux, address);
      USB.print("\r\n**write new key, state: ");
      USB.print(state, HEX);
      if (state == 1)
      {
        USB.print(" --> correct key change to the original");
      }
      else
      {
        USB.print(" --> ** ERROR: keys not changed ** ");
      }
      // ###### check block 3/7/11/15
      // **** authenticate with the original key A
      state = RFID13.authenticate(CardID, address , keyOld);
      USB.print("\r\n**Authentication mem block  : ");
      USB.print(state, HEX);
      if (state == 1)
      {
        USB.print(" --> success to authenticate with original key. All changes undone");
      }
      else
      {
        USB.print(" --> ** ERROR: bad authentication ** ");
      }
    }
  delay(3000); // wait some time in each loop
}
