 /*  
  *  ------ Read All Blocks --------
  *  
  *  Explanation: This sketch reads all the blocks in a RFID card. It is
  *  supposed that all the sectors have the same key.
  *  
  *  Copyright (C) 2012 Libelium Comunicaciones Distribuidas S.L.
  *  http://www.libelium.com
  *  
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *  
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  *  GNU General Public License for more details.
  *  
  *  You should have received a copy of the GNU General Public License
  *  along with this program. If not, see <http://www.gnu.org/licenses/>.
  *  
  *  Version:                0.1
  *  Design:                 David Gascon
  *  Implementation:         Javier Solobera, Ahmad Saad
  */



// stores the status of the executed command:
short state;
// auxiliar buffer:
unsigned char aux[16];
// stores the UID (unique identifier) of a card:
unsigned char CardID[4];
// stores the key or password:
unsigned char keyAccess[] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};



void setup()
{  
  // switchs on the module, type B, and asigns the socket
  RFID13.ON(SOCKET0, B);
  delay(1000);
  
  USB.begin();
  USB.print("\n");
  USB.println("NFC @ 13.56 MHz module started");
}



void loop()
{
  USB.print("\r\n++++++++++++++++++++++++++++++++++++");
  // **** init the RFID reader
  state = RFID13.init(CardID, aux);
  if (aux[0] == aux[1])  // if so, there is no card on the EM field
  {
    USB.print("\n");
    USB.print("\r\nRequest error-no card found");
  }
  else  // a card was found
  {
    USB.print("\n");
    
    USB.print("\r\nRequest|Answer: ");
    RFID13.print(aux, 2); // show the ATQ
    
    USB.print("\r\nAnticollSelect: ");
    USB.print(state, HEX); // show the status of the executed init command (should be 0)
    USB .print("|Answer: ");
    
    // show the UID (Unique IDentifier) of the read card (4 bytes)
    RFID13.print(CardID, 16);
    
    // we will do the same actions in blocks from 0 to 3 (which is sector 1)
    for (int n=0; n<64; n++)
    {
      // **** authenticate the key A of each sector
      if (n % 4 == 0) // only one authentication per sector is needed
      {
        state = RFID13.authenticate(CardID, n , keyAccess); // it is supposed all the blocks have the same key
        USB.print("\n");
        USB.print("\r\nAuthentication block");
        USB.print(n, DEC); // show the number of the block [n = 0..63 in a 1 kB RFID card]
        USB.print(": ");
        USB.print(state, HEX); // show the state of the executed authentication command
        delay(1);
      }
      // **** if passed authentication in the sector, we will be able to read the data in its blocks
      state = RFID13.readData(n, aux);
      USB.print("\n");
      USB.print("\r\nRead block ");
      USB.print(n, DEC); // show the number of the block [n = 0..63]
      USB.print(": ");
      USB.print(state, HEX); // show the state of the executed read command
      USB.print("   ");
      
      if (state == 1) // if the read command was successful, we show the data (16 bytes)
      {
        USB.print(" | ");
        for (int i=0; i<16; i++)
        {
          if (aux[i] <=  B01111)
          {
            USB.print(" ");
          }
          USB.print(aux[i], HEX); // print the 16 bytes of the block 'n'
          USB.print(" ");
        }
        USB.print("  ");
        if (n==0)
          USB.print("<--UID(first 4 bytes)");
        else if (((n+1) % 4) == 0)
          USB.print("<--Trailer block");
          USB.print("\n");
      }
      delay(1);
    }
    USB.println();
  }
  delay(2000); // wait 1 second in each loop
}
