/*  
 *  ------ [SD_2] - Write data to a SD file -------- 
 *  
 *  Explanation: Turn on the SD card. Write data in SD file
 *  indicating the position to write.
 *  
 *  Copyright (C) 2012 Libelium Comunicaciones Distribuidas S.L. 
 *  http://www.libelium.com 
 *  
 *  This program is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  This program is distributed in the hope that it will be useful, 
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 *  GNU General Public License for more details. 
 *  
 *  You should have received a copy of the GNU General Public License 
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. 
 *  
 *  Version:           0.1 
 *  Design:            David Gascón 
 *  Implementation:    Yuri Carmona
 */

// define file name: MUST be 8.3 SHORT FILE NAME
char* filename="FILE2.TXT";

// define hexadecimal data
uint8_t data[10]={0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0xAA,0xAA};

void setup()
{
  // open USB port
  USB.begin();
  USB.println(F("SD_2 example"));
  
  // Set SD ON
  SD.ON();
    
  // Delete file
  if(SD.del(filename)) 
  {
    USB.println(F("file deleted"));
  }
  else 
  {
    USB.println(F("file NOT deleted"));  
  }
    
  // Create file
  if(SD.create(filename))
  {
    USB.println(F("file created"));
  }
  else 
  {
    USB.println(F("file NOT created"));  
  } 
}


void loop()
{  
  // 1 - It writes “hello” in file at position 0
  if(SD.writeSD(filename,"hello", 0)) USB.println(F("1.write OK"));
  else USB.println(F("1.write failed"));  
  
  // show file
  showFile();
  delay(1000);
  

  
  // 2 - It writes “goodbye” in file at position 5
  if(SD.writeSD(filename,"goodbye", 5)) USB.println(F("2.write OK"));
  else USB.println(F("2.write failed"));  
  
  // show file
  showFile();
  delay(1000);



  // 3 - It writes “hel” in file at position 10 indicating 3-byte length
  if(SD.writeSD(filename,"hello", 10, 3)) USB.println(F("3.write OK"));
  else USB.println(F("3.write failed"));   
 
   // show file
  showFile();
  delay(1000);
   
   
   
  // 4 - It writes hexadecimal data indicating the length to write
  if(SD.writeSD(filename, data, 0, 3)) USB.println(F("4.write OK"));
  else USB.println(F("4.write failed"));  
  
  // show file
  showFile();
  delay(1000);
   
   
   
  // 5 - It writes hexadecimal data
  // End of data array is defined by 0xAAAA
  if(SD.writeSD(filename, data, 0)) USB.println("5.write ok");
  else USB.println("5.write failed");

  // show file
  showFile();   
  USB.println(F("\n***************************"));
  delay(10000);
}




/*
 * Local Function in order to show the file contents
 *
 */
void showFile()
{
  // show file
  USB.println(F("Show file:"));      
  USB.println(F("-------------------"));
  USB.println(SD.catln(filename, 0, SD.numln(filename) ));
  USB.println(F("-------------------"));
}
