/*  
 *  ------ [SD_3] - Append data to SD file -------- 
 *  
 *  Explanation: Turn on the SD card. Append data t the end of
 *  the SD file. 
 *  
 *  Copyright (C) 2012 Libelium Comunicaciones Distribuidas S.L. 
 *  http://www.libelium.com 
 *  
 *  This program is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  This program is distributed in the hope that it will be useful, 
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 *  GNU General Public License for more details. 
 *  
 *  You should have received a copy of the GNU General Public License 
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. 
 *  
 *  Version:           0.1 
 *  Design:            David Gascón 
 *  Implementation:    Yuri Carmona
 */

// define file name: MUST be 8.3 SHORT FILE NAME
char* filename="FILE3.TXT";

// define an example string
char* data="hello"; 



void setup()
{
  // open USB port
  USB.begin();
  USB.println(F("SD_3 example"));
  
  // Set SD ON
  SD.ON();
    
  // Delete file
  if(SD.del(filename)) 
  {
    USB.println(F("file deleted"));
  }
  else 
  {
    USB.println(F("file NOT deleted"));  
  }
    
  // Create file
  if(SD.create(filename))
  {
    USB.println(F("file created"));
  }
  else 
  {
    USB.println(F("file NOT created"));  
  } 
}


void loop()
{ 
  
  // 1 - It appends “he” in file indicating 2-byte length
  if(SD.append(filename, data, 2)) USB.println(F("\n1.append ok"));
  else USB.println(F("\n1.append error"));
  
  // show file
  showFile();
  delay(1000);
  
  
  // 2 - It appends “hello” at the end of the file
  if(SD.append(filename, data)) USB.println(F("\n2.append ok"));
  else USB.println(F("\n2.append error"));
  
  // show file
  showFile();
  delay(1000);
  
  
  // 3 - It appends data to file adding an End Of Line at the end of the string
  if(SD.appendln(filename,"first\nsecond\nthird"))  USB.println(F("\n3.append ok"));
  else USB.println(F("\n3.append error"));
  
  // show file
  showFile();
  delay(1000); 
 
}


/*
 * Local Function in order to show the file contents
 *
 */
void showFile()
{
  // show file
  USB.println(F("Show file:"));      
  USB.println(F("-------------------"));
  USB.println(SD.catln(filename, 0, SD.numln(filename) ));
  USB.println(F("-------------------"));
}
