/*
 *  ------Waspmote WIFI_07 Example--------
 *
 *  Explanation: This example shows how to join to an AP in AUTO BEST mode.
 *  In this mode the module tries to join any access point with security 
 *  matching the stored authentication mode. In this particular example,
 *  the module tries to match an OPEN network so no authentication is 
 *  previously set.
 *  
 *  Copyright (C) 2013 Libelium Comunicaciones Distribuidas S.L.
 *  http://www.libelium.com
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Version:                0.2
 *  Design:                 David Gascón
 *  Implementation:         Yuri Carmona
 */


// choose socket (SELECT USER'S SOCKET)
///////////////////////////////////////
uint8_t socket=SOCKET1;
///////////////////////////////////////


void setup()
{
  USB.begin();
  USB.println("Example WiFi 07");
  
  // Switch ON the WiFi module on the desired socketWIFI module on the desired socket.
  if( WIFI.ON(socket) == 1 )
  {    
    USB.println("WiFi switched ON");
  }
  else
  {
    USB.println("WiFi did not initialize correctly");
  }
  
  // 1. Configure the transport protocol (UDP, TCP, FTP, HTTP...)
  WIFI.setConnectionOptions(UDP);
  // 2. Configure the way the modules will resolve the IP address.
  WIFI.setDHCPoptions(DHCP_ON); 
  // *** AUTO-BEST *** 
  // 3. Configure the Authentication mode of the auto-join. 
  WIFI.setAutojoinAuth(OPEN); 
  // 4. Save Data to module's memory
  WIFI.storeData();
  
  USB.println("WiFi setup done");
}

void loop()
{
  // Switch on the WiFi module
  WIFI.ON(socket); 
  
  // Configure how to connect the AP. 
  if(WIFI.setJoinMode(AUTO_BEST)) 
  {
    USB.println("joined AP");

    // Displays Access Point status.
    USB.println("\n----------------------");
    USB.println("AP Status:");
    USB.println("----------------------");
    WIFI.getAPstatus();

    // Displays IP settings.
    USB.println("\n----------------------");
    USB.println("IP Settings:");
    USB.println("----------------------");
    WIFI.getIP();
    USB.println();
    
    // Call the function that needs a connection. 
    WIFI.resolve("www.libelium.com"); 
  }
  else
  {
    USB.println("not joined");
  }
  
  // Switch off the WiFi module
  WIFI.OFF();
  USB.println("***************************");
}


