/*
 *  ------Waspmote WIFI_08 Example--------
 *
 *  Explanation: This example shows how to join to an AP in AUTO STORE mode.
 *  In this mode the module tries to join the access point that matches the 
 *  stored SSID, passkey and channel. In this particular example, the module 
 *  tries to match an OPEN network so no authentication is previously set.
 *  On the other hand, channel and ESSID are stored.
 *  
 *  Copyright (C) 2013 Libelium Comunicaciones Distribuidas S.L.
 *  http://www.libelium.com
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Version:                0.2
 *  Design:                 David Gascón
 *  Implementation:         Yuri Carmona
 */
 
// Include WIFI library 
#include <WaspWIFI.h>

// choose socket (SELECT USER'S SOCKET)
///////////////////////////////////////
uint8_t socket=SOCKET1;
///////////////////////////////////////

// WiFi AP settings (CHANGE TO USER'S AP)
/////////////////////////////////
#define ESSID "libelium_AP"
/////////////////////////////////

void setup()
{
  USB.begin();
  USB.println(F("Example WiFi 08"));
  
  // Switch on the WIFI module on the desired socket.
  WIFI.ON(socket);
  
  // 1. Configure the transport protocol (UDP, TCP, FTP, HTTP...)
  WIFI.setConnectionOptions(UDP);
  // 2. Configure the way the modules will resolve the IP address.
  WIFI.setDHCPoptions(DHCP_ON);  
  // *** AUTO-STORE *** 
  // 3. Configure the authentication mode of the auto-join. 
  WIFI.setAutojoinAuth(OPEN); 
  // 4. Sets the name of the AP we want to join. 
  WIFI.setESSID(ESSID); 
  // 5. Store Values
  WIFI.storeData();
}

void loop()
{
  WIFI.ON(socket);
  
  // 4. Configure how to connect the AP. 
  if (WIFI.setJoinMode(AUTO_STOR)) 
  {
    USB.println(F("joined AP"));

    // Displays Access Point status.
    USB.println(F("\n----------------------"));
    USB.println(F("AP Status:"));
    USB.println(F("----------------------"));
    WIFI.getAPstatus();

    // Displays IP settings.
    USB.println(F("\n----------------------"));
    USB.println(F("IP Settings:"));
    USB.println(F("----------------------"));
    WIFI.getIP();
    USB.println();
    
    // 4.1 Call the function that needs a connection. 
    WIFI.resolve("www.libelium.com"); 
  }
  else
  {
    USB.println(F("not joined"));
  }
  
  
  WIFI.OFF();
  USB.println(F("**************************"));
}
