/*
 *  ------Waspmote WIFI_26 Example--------
 *
 *  Explanation: This example shows how to send a HTTP get request 
 *  message. 
 *
 *  Copyright (C) 2013 Libelium Comunicaciones Distribuidas S.L.
 *  http://www.libelium.com
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Version:                0.2
 *  Design:                 David Gascón
 *  Implementation:         Joaquin Ruiz
 */



// choose socket (SELECT USER'S SOCKET)
///////////////////////////////////////
uint8_t socket=SOCKET1;
///////////////////////////////////////

// WiFi AP settings (CHANGE TO USER'S AP)
/////////////////////////////////
#define ESSID "libelium_AP"
#define AUTHKEY "password"
/////////////////////////////////

// WEB server settings 
/////////////////////////////////
#define IP_ADDRESS "10.10.10.1"
#define PORT 21
/////////////////////////////////


// define variable for communication status
uint8_t status;
uint8_t counter=0;
char sentence[100];
unsigned long previous;


void setup()
{
  USB.begin();
  USB.println("Example Wifi 26");
  
  if( WIFI.ON(socket) == 1 )
  {    
    USB.println(F("WiFi switched ON"));
  }
  else
  {
    USB.println(F("WiFi did not initialize correctly"));
  }

  //WIFI.resetValues();

  // 1. Configure the transport protocol (UDP, TCP, FTP, HTTP...)
  WIFI.setConnectionOptions(HTTP|CLIENT_SERVER);
  // 2. Configure the way the modules will resolve the IP address.
  WIFI.setDHCPoptions(DHCP_ON);
  // 3. Configure how to connect the AP 
  WIFI.setJoinMode(MANUAL);   
  // 4. Set the AP authentication key
  WIFI.setAuthKey(WPA1,AUTHKEY); 
  // 5. Save Data to module's memory
  WIFI.storeData();

}


void loop()
{ 
  // switch on the WiFi module  
  WIFI.ON(socket);
  
  // If it is manual, call join giving the name of the AP     
  if( WIFI.join(ESSID) )
  { 
    USB.println(F("Joined"));

    /////////////////////////////////////////// 
    // Create the sentence
    /////////////////////////////////////////// 
    RTC.ON();
    RTC.getTime();
    counter++;
    sprintf(sentence,"GET$/tests/query.php?FRAME=counter:%u.TIME:%02u:%02u:%02u",counter,RTC.hour,RTC.minute,RTC.second);
    
    USB.print("sentence:");
    USB.print(sentence);
    
    /////////////////////////////////////////// 
    // Send the HTTP get/post query (specifying the IP address)
    /////////////////////////////////////////// 
    status=WIFI.getURL(IP, IP_ADDRESS, sentence);

    if( status == 1)
    {
      USB.println(F("\nHTTP query OK"));
    }
    else
    {
      USB.println(F("\nHTTP query ERROR"));
      counter--;
    }
  } 
  else
  {
    USB.println(F("NOT joined"));
  }

  // switch off module 
  WIFI.OFF();  
  
  USB.begin();
  USB.println(F("***************************"));  
  delay(1000);
} 

