/*  
 *  -------------- GPRS_19 - Getting URL  -------------- 
 *  
 *  Explanation: This example shows how to get an URL
 *  
 *  Copyright (C) 2013 Libelium Comunicaciones Distribuidas S.L. 
 *  http://www.libelium.com 
 *  
 *  This program is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  This program is distributed in the hope that it will be useful, 
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 *  GNU General Public License for more details. 
 *  
 *  You should have received a copy of the GNU General Public License 
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. 
 *  
 *  Version:           0.3
 *  Design:            David Gascón 
 *  Implementation:    Alejandro Gállego
 */


int answer;
char aux_str[150];
unsigned int counter = 0;
float temp;
int d1;      
float f2;
int d2;  

void setup()
{
    USB.println(F("---******************************************************************************---"));
    USB.println(F("GET method to the libelium's test url..."));
    USB.println(F("You can use this php to test the HTTP connection of the module."));
    USB.println(F("The php returns the parameters that the user sends with the URL."));
    USB.println(F("In this case the loop counter (counter) and the RTC temperature (temp)."));
    USB.println(F("The syntax to add parameters is below:"));
    USB.println(F("test-get-post.php?parameter1=value1&parameter2=value2&...&last_parameter=last_value"));
    USB.println(F("---******************************************************************************---"));
    
}

void loop()
{

    // setup for Serial port over USB:
    USB.begin();
    USB.println(F("USB port started..."));

    // activates the GPRS_Pro module:
    answer = GPRS_Pro.ON(); 
    if ((answer == 1) || (answer == -3))
    { 
        USB.println(F("GPRS_Pro module ready..."));
    
        // sets pin code:
        USB.println(F("Setting PIN code..."));
        // **** must be substituted by the SIM code
        if (GPRS_Pro.setPIN("****") == 1) 
        {
            USB.println(F("PIN code accepted"));
        }
        else
        {
            USB.println(F("PIN code incorrect"));
        }
    
       // waits for connection to the network:
        answer = GPRS_Pro.check(180);    
        if (answer == 1)
        {
             
            USB.println(F("GPRS_Pro module connected to the network..."));
        
            // configures GPRS connection for HTTP or FTP applications:
            answer = GPRS_Pro.configureGPRS_HTTP_FTP(1);
            if (answer == 1)
            {
                USB.print(F("Get the URL..."));
                RTC.ON();
                temp = RTC.getTemperature();
                RTC.OFF();
                d1 = temp;            // Get the integer part (678).
                f2 = temp - d1;     // Get fractional part (678.0123 - 678 = 0.0123).
                d2 = f2 * 100;   // Turn into integer (123).
                
                sprintf(aux_str, "http://pruebas.libelium.com/test-get-post.php?counter=%d&temp=%d.%02d", counter, d1, d2);
                answer = GPRS_Pro.readURL(aux_str, 1);
                // gets URL from the solicited URL
                if ( answer == 1)
                {
                    USB.println(F("Done"));  
                    USB.println(GPRS_Pro.buffer_GPRS);
                }
                else if (answer < -9)
                {
                    USB.print(F("Failed. Error code: "));
                    USB.println(answer, DEC);
                    USB.print(F("CME error code: "));
                    USB.println(GPRS_Pro.CME_CMS_code, DEC);
                }
                else 
                {
                    USB.print(F("Failed. Error code: "));
                    USB.println(answer, DEC);
                }
                       
            }
            else
            {
                USB.println(F("Configuration 1 failed. Error code: "));
                USB.println(answer, DEC);
            }
        }
        else
        {
            USB.println(F("GPRS_Pro module cannot connect to the network"));     
        }
    }
    else
    {
        USB.println(F("GPRS_Pro module not ready"));    
    }

    GPRS_Pro.OFF(); // powers off the GPRS_Pro module
    counter++;
    USB.println(F("Sleeping..."));

    // activates the RTC:
    RTC.ON();

    // sleeps one hour
    PWR.deepSleep("00:01:00:00", RTC_OFFSET, RTC_ALM1_MODE1, ALL_OFF);

    // deactivates the RTC:
    RTC.OFF();

}




