/*
 *  ------Waspmote RTC Interruption Example--------
 *
 *  Explanation: This example shows how to manage RTC Interruption
 *
 *  Copyright (C) 2009 Libelium Comunicaciones Distribuidas S.L.
 *  http://www.libelium.com
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Version:                0.1
 *  Design:                 David Gascón
 *  Implementation:         Alberto Bielsa
 */
 
 
void setup()
{  
  // Powers RTC up, init I2C bus and read initial values
  USB.begin();
  USB.println("Init RTC");
  RTC.ON();      
}


void loop(){

  // Setting time
  RTC.setTime("09:10:20:03:17:35:00");
  USB.print("Time: ");
  USB.println(RTC.getTime());
  
  // Setting and getting Alarms
  RTC.setAlarm1("20:17:35:30",RTC_ABSOLUTE,RTC_ALM1_MODE2);
  USB.print("Alarm1: ");
  USB.println(RTC.getAlarm1());  
  
  // Setting Waspmote to Low-Power Consumption Mode
  PWR.sleep(ALL_OFF);
  
  // After setting Waspmote to power-down, UART is closed, so it
  // is necessary to open it again
  USB.begin();
  
  // Waspmote wakes up at '17:35:30'
  if( intFlag & RTC_INT )
  {
    intFlag &= ~(RTC_INT); // Clear flag
    Utils.blinkLEDs(1000); // Blinking LEDs
    Utils.blinkLEDs(1000); // Blinking LEDs
  }    
  
  RTC.setAlarm2("20:17:36",RTC_ABSOLUTE,RTC_ALM2_MODE2);
  USB.print("Alarm2: ");  
  USB.println(RTC.getAlarm2());  
  
  // Setting Waspmote to Low-Power Consumption Mode
  PWR.sleep(ALL_OFF);
  
  // After setting Waspmote to power-down, UART is closed, so it
  // is necessary to open it again
  USB.begin();
  
  // Waspmote wakes up at '17:36'
  if( intFlag & RTC_INT )
  {
    intFlag &= ~(RTC_INT); // Clear flag
    Utils.blinkLEDs(1000); // Blinking LEDs
    Utils.blinkLEDs(1000); // Blinking LEDs
  }    
}

