/*  
 *  ------ [SD_08] - Change current working directory -------- 
 *  
 *  Explanation: Turn on the SD card. Move from one to another directory
 *  in order to manage files directly in each folder instead of using
 *  complete paths.
 *  
 *  Copyright (C) 2012 Libelium Comunicaciones Distribuidas S.L. 
 *  http://www.libelium.com 
 *  
 *  This program is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  This program is distributed in the hope that it will be useful, 
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 *  GNU General Public License for more details. 
 *  
 *  You should have received a copy of the GNU General Public License 
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. 
 *  
 *  Version:           0.1 
 *  Design:            David Gascón 
 *  Implementation:    Yuri Carmona
 */

// define folder and file names to be created: MUST be 8.3 SHORT FILE NAME
char* path="/fold1/fold2/fold3";
char* file1="file1";
char* file2="file2";
char* file3="file3";


void setup()
{
  // open USB port
  USB.begin();
  USB.println(F("SD_08 example"));

  // Set SD ON
  SD.ON();

  // delete all contained files in 'path'
  SD.rmRfDir("/fold1");

  // create path
  SD.mkdir(path);

}

void loop()
{ 
  // Starting current working directory is Root directory  

  /////////////////////////////////////////////////////////
  // 1. change directory to 'fold3'
  /////////////////////////////////////////////////////////
  if(SD.cd(path))
  { 
    USB.println(F("\ninside /fold1/fold2/fold3:"));
  }
  else
  { 
    USB.println("cd failed");
  }

  /////////////////////////////////////////////////////////
  // 2. create a file inside /fold1/fold2/fold3/
  SD.create( file3 );
  /////////////////////////////////////////////////////////
  
  ///////////////////////////////////////////////////////// 
  // 3. list current working directory: fold3
  /////////////////////////////////////////////////////////
  USB.println(F("-----------------------------"));
  SD.ls( LS_SIZE );  
  USB.println(F("-----------------------------"));
  delay(1000);

  /////////////////////////////////////////////////////////
  // 4. Go one directory up to /fold1/fold2/  
  /////////////////////////////////////////////////////////
  if(SD.cd(".."))
  {
    USB.println("\ninside /fold1/fold2/:");
  }
  else
  { 
    USB.println("cd failed");  
  }

  /////////////////////////////////////////////////////////
  // 5. create a file inside /fold1/fold2/
  /////////////////////////////////////////////////////////
  SD.create(file2);

  /////////////////////////////////////////////////////////
  // 6. list current working directory: fold2
  /////////////////////////////////////////////////////////  
  USB.println(F("-----------------------------"));
  SD.ls( LS_SIZE );  
  USB.println(F("-----------------------------"));
  delay(1000);

  /////////////////////////////////////////////////////////
  // 7 - go one directory up to /fold1/
  /////////////////////////////////////////////////////////
  if(SD.cd(".."))
  {
    USB.println("\ninside /fold1/");
  }
  else
  { 
    USB.println("cd failed");  
  }

  /////////////////////////////////////////////////////////
  // 8. create a file inside /fold1/
  /////////////////////////////////////////////////////////
  SD.create(file1);

  ///////////////////////////////////////////////////////// 
  // 9. list current working directory: fold1
  /////////////////////////////////////////////////////////
  USB.println(F("-----------------------------"));
  SD.ls( LS_SIZE );  
  USB.println(F("-----------------------------"));
  delay(1000);
  
  
  /////////////////////////////////////////////////////////
  // 11. go one directory up to ROOT directory
  /////////////////////////////////////////////////////////
  if(SD.cd(".."))
  {
    USB.println("\ninside ROOT directory:");
  }
  else
  {
    USB.println("cd failed");  
  }

  /////////////////////////////////////////////////////////
  // 12. list Root directory
  /////////////////////////////////////////////////////////
  USB.println(F("###############################"));
  SD.ls( LS_R | LS_SIZE );    
  USB.println(F("###############################"));
  
  
  delay(5000);
}


